package com.example.wtf.controller;

import com.example.wtf.exception.ResourceNotFound;
import com.example.wtf.model.Image;
import com.example.wtf.repository.ImageRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.methodOn;

@RestController
@RequestMapping("/images")
public class ImageController {

    @Autowired
    private ImageRepository repository;

    @GetMapping("/get")
    public @ResponseBody CollectionModel<EntityModel<Image>> getImages() {
        List<EntityModel<Image>> images = new ArrayList<>();

        for (Image image : repository.findAll()) {
            EntityModel<Image> imageResource = EntityModel.of(
                    image,
                    linkTo(methodOn(ImageController.class).getImage(image.getId())).withSelfRel(),
                    linkTo(methodOn(ImageController.class).updateImage(image.getId(), new Image())).withRel("Update Image"),
                    linkTo(methodOn(ImageController.class).deleteImage(image.getId())).withRel("Delete Image")
            );
            images.add(imageResource);
        }

        return CollectionModel.of(
                images,
                linkTo(methodOn(ImageController.class).addImage(new Image())).withRel("Add Image")
        );
    }

    @GetMapping("/get/{id}")
    public @ResponseBody EntityModel<Image> getImage(@PathVariable Long id) {
        Image image = repository.findById(id)
                .orElseThrow(() -> new ResourceNotFound("Image not found"));

        return EntityModel.of(
                image,
                linkTo(methodOn(ImageController.class).updateImage(image.getId(), new Image())).withRel("Update Image"),
                linkTo(methodOn(ImageController.class).deleteImage(image.getId())).withRel("Delete Image"),
                linkTo(methodOn(ImageController.class).getImages()).withRel("Images")
        );
    }

    @PostMapping("/add")
    public @ResponseBody EntityModel<Image> addImage(@RequestBody Image image) {
        repository.save(image);
        return EntityModel.of(
                image,
                linkTo(methodOn(ImageController.class).getImage(image.getId())).withSelfRel(),
                linkTo(methodOn(ImageController.class).updateImage(image.getId(), new Image())).withRel("Update Image"),
                linkTo(methodOn(ImageController.class).deleteImage(image.getId())).withRel("Delete Image"),
                linkTo(methodOn(ImageController.class).getImages()).withRel("Images")
        );
    }

    @PutMapping("/update/{id}")
    public @ResponseBody EntityModel<Image> updateImage(@PathVariable Long id,
                                                        @RequestBody Image update) {
        Image image = repository.findById(id)
                .orElseThrow(() -> new ResourceNotFound("Image not found"));

        if (update.getPath() != null && !update.getPath().isEmpty()) {
            image.setPath(update.getPath());
        }

        repository.save(image);

        return EntityModel.of(
                image,
                linkTo(methodOn(ImageController.class).getImage(id)).withSelfRel(),
                linkTo(methodOn(ImageController.class).deleteImage(id)).withRel("Delete Image"),
                linkTo(methodOn(ImageController.class).getImages()).withRel("Images")
        );
    }

    @DeleteMapping("delete/{id}")
    public @ResponseBody EntityModel<Image> deleteImage(@PathVariable Long id) {
        Image image = repository.findById(id)
                .orElseThrow(() -> new ResourceNotFound("Image not found"));

        repository.delete(image);

        return EntityModel.of(
                image,
                linkTo(methodOn(ImageController.class).getImages()).withRel("Images")
        );
    }
}