package com.example.wtf.controller;

import com.example.wtf.exception.ResourceNotFound;
import com.example.wtf.model.Question;
import com.example.wtf.repository.QuestionRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.methodOn;

@RestController
@RequestMapping("/questions")
public class QuestionController {
    @Autowired
    private QuestionRepository repository;

    @GetMapping("/get")
    public @ResponseBody CollectionModel<EntityModel<Question>> getQuestions() {
        List<EntityModel<Question>> questions = new ArrayList<>();

        for (Question question : repository.findAll()) {
            EntityModel<Question> questionResource = EntityModel.of(
                    question,
                    linkTo(methodOn(QuestionController.class).getQuestion(question.getId())).withSelfRel(),
                    linkTo(methodOn(QuestionController.class).updateQuestion(question.getId(), new Question())).withRel("Update Question"),
                    linkTo(methodOn(QuestionController.class).deleteQuestion(question.getId())).withRel("Delete Question")
            );
            questions.add(questionResource);
        }

        return CollectionModel.of(
                questions,
                linkTo(methodOn(QuestionController.class).addQuestion(new Question())).withRel("Add Question")
        );
    }

    @GetMapping("/get/{id}")
    public @ResponseBody EntityModel<Question> getQuestion(@PathVariable Long id) {
        Question question = repository.findById(id)
                .orElseThrow(() -> new ResourceNotFound("Question not found"));

        return EntityModel.of(
                question,
                linkTo(methodOn(QuestionController.class).updateQuestion(question.getId(), new Question())).withRel("Update Question"),
                linkTo(methodOn(QuestionController.class).deleteQuestion(question.getId())).withRel("Delete Question"),
                linkTo(methodOn(QuestionController.class).getQuestions()).withRel("Questions")
        );
    }

    @PostMapping("/add")
    public @ResponseBody EntityModel<Question> addQuestion(@RequestBody Question question) {
        repository.save(question);
        return EntityModel.of(
                question,
                linkTo(methodOn(QuestionController.class).getQuestion(question.getId())).withSelfRel(),
                linkTo(methodOn(QuestionController.class).updateQuestion(question.getId(), new Question())).withRel("Update Question"),
                linkTo(methodOn(QuestionController.class).deleteQuestion(question.getId())).withRel("Delete Question"),
                linkTo(methodOn(QuestionController.class).getQuestions()).withRel("Questions")
        );
    }

    @PutMapping("/update/{id}")
    public @ResponseBody EntityModel<Question> updateQuestion(@PathVariable Long id,
                                                              @RequestBody Question update) {
        Question question = repository.findById(id)
                .orElseThrow(() -> new ResourceNotFound("Question not found"));

        if (update.getContent() != null && !update.getContent().isEmpty()) {
            question.setContent(update.getContent());
        }
        if (update.getAnswerA() != null && !update.getAnswerA().isEmpty()) {
            question.setAnswerA(update.getAnswerA());
        }
        if (update.getAnswerB() != null && !update.getAnswerB().isEmpty()) {
            question.setAnswerB(update.getAnswerB());
        }
        if (update.getAnswerC() != null && !update.getAnswerC().isEmpty()) {
            question.setAnswerC(update.getAnswerC());
        }
        if (update.getAnswerD() != null && !update.getAnswerD().isEmpty()) {
            question.setAnswerD(update.getAnswerD());
        }
        if (update.getcAnswer() != null && !update.getcAnswer().isEmpty()) {
            question.setcAnswer(update.getcAnswer());
        }

        repository.save(question);

        return EntityModel.of(
                question,
                linkTo(methodOn(QuestionController.class).getQuestion(id)).withSelfRel(),
                linkTo(methodOn(QuestionController.class).deleteQuestion(id)).withRel("Delete Question"),
                linkTo(methodOn(QuestionController.class).getQuestions()).withRel("Questions")
        );
    }

    @DeleteMapping("delete/{id}")
    public @ResponseBody EntityModel<Question> deleteQuestion(@PathVariable Long id) {
        Question question = repository.findById(id)
                .orElseThrow(() -> new ResourceNotFound("Question not found"));

        repository.delete(question);

        return EntityModel.of(
                question,
                linkTo(methodOn(QuestionController.class).getQuestions()).withRel("Questions")
        );
    }
}
